# -*- coding: utf-8 -*- vim: ts=8 sts=4 sw=4 si et tw=79
"""\
AJAX navigation support for Plone sites
"""

__author__ = "Tobias Herp <tobias.herp@visaplan.com>"
PROJECTNAME = 'visaplan.plone.ajaxnavigation'
